package com.bpcl.lpg.core.service.impl;

import org.slf4j.LoggerFactory;
import java.text.MessageFormat;
import org.apache.commons.lang.StringUtils;
import java.util.Objects;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import java.net.http.HttpResponse;
import com.bpcl.lpg.facades.data.LpgRequestData;
import com.bpcl.lpg.core.service.BpclHttpClientService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.slf4j.Logger;
import com.bpcl.lpg.core.service.LpgConnectionService;

public class LpgRSPConnectionServiceImpl implements LpgConnectionService
{
    private static final Logger LOG;
    private ConfigurationService configurationService;
    private BpclHttpClientService bpclHttpClientService;
    
    public HttpResponse<String> getResponse(final LpgRequestData lpgRequestData, final String requestId, final String endPointName) throws JsonProcessingException {
        this.validateParameter(lpgRequestData, endPointName);
        final String endPoint = this.getEndPoint(endPointName, requestId);
        final String jsonBody = this.getRequestJson(lpgRequestData, endPoint);
        return this.getBpclHttpClientService().executePostMethod(endPoint, jsonBody, endPointName);
    }
    
    private String getRequestJson(final LpgRequestData lpgRequestData, final String endPointName) throws JsonProcessingException {
        final ObjectMapper objectMapper = new ObjectMapper();
        final String json = objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(lpgRequestData);
        return objectMapper.writeValueAsString(lpgRequestData);
    }
    
    private void validateParameter(final LpgRequestData lpgRequestData, final String endPointName) {
        Objects.requireNonNull(lpgRequestData, "LPG Request data should not be null");
        Objects.requireNonNull(endPointName, "End point name should not be null");
    }
    
    private String getEndPoint(final String endPointName, final String requestId) {
        final String endpointUri = this.getConfigurationService().getConfiguration().getString(endPointName);
        Objects.requireNonNull(endpointUri, "Endpoint should not be null");
        return StringUtils.isBlank(requestId) ? endpointUri : MessageFormat.format(endpointUri, requestId);
    }
    
    public ConfigurationService getConfigurationService() {
        return this.configurationService;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public BpclHttpClientService getBpclHttpClientService() {
        return this.bpclHttpClientService;
    }
    
    public void setBpclHttpClientService(final BpclHttpClientService bpclHttpClientService) {
        this.bpclHttpClientService = bpclHttpClientService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LpgRSPConnectionServiceImpl.class);
    }
}
